@@ -45,7 +45,7 @@
 #define EVP_MAX_BLOCK_LENGTH 32	/* some older openssl versions need this */
 #endif
 #else
-#define EVP_MAX_BLOCK_LENGTH 0  /* FIXME: this is ugly, but needed */
+#define EVP_MAX_BLOCK_LENGTH 0	/* FIXME: this is ugly, but needed */
 #endif				/* HAVE_LIBCRYPTO */
 #include <pam_mount.h>
 #include <libgen.h>
@@ -68,11 +68,11 @@ static int hash_authtok(FILE * const fp, const EVP_CIPHER * const cipher,
 	unsigned char salt[PKCS5_SALT_LEN];
 	char magic[sizeof "Salted__" - 1];
 
-	assert(fp != NULL);		/* FIXME: check if valid, open file */
-	assert(cipher != NULL);		/* FIXME: check if cipher is valid OpenSSL cipher */
+	assert(fp != NULL);	/* FIXME: check if valid, open file */
+	assert(cipher != NULL);	/* FIXME: check if cipher is valid OpenSSL cipher */
 	assert(authtok != NULL);
-	assert(hash != NULL);		/* FIXME: check hash is big enough? */
-	assert(iv != NULL);		/* FIXME: check iv is big enough? */
+	assert(hash != NULL);	/* FIXME: check hash is big enough? */
+	assert(iv != NULL);	/* FIXME: check iv is big enough? */
 
 	if ((fread(magic, 1, sizeof "Salted__" - 1, fp) !=
 	     sizeof "Salted__" - 1)
@@ -114,7 +114,7 @@ static int hash_authtok(FILE * const fp, const EVP_CIPHER * const cipher,
  *       pt_fs_key may contain trailing garbage; use pt_fs_key_len
  */
 static int
-decrypted_key(char *const pt_fs_key, size_t *const pt_fs_key_len,
+decrypted_key(char *const pt_fs_key, size_t * const pt_fs_key_len,
 	      const char *const fs_key_path,
 	      const char *const fs_key_cipher, const char *const authtok)
 {
@@ -139,7 +139,7 @@ decrypted_key(char *const pt_fs_key, size_t *const pt_fs_key_len,
 	memset(pt_fs_key, 0x00, MAX_PAR + EVP_MAX_BLOCK_LENGTH);
 	OpenSSL_add_all_ciphers();
 	EVP_CIPHER_CTX_init(&ctx);
-	if (! (fs_key_fp = fopen(fs_key_path, "r"))) {
+	if (!(fs_key_fp = fopen(fs_key_path, "r"))) {
 		l0g("pam_mount: error opening %s\n", fs_key_path);
 		ret = 0;
 		goto _return_no_close;
@@ -150,7 +150,8 @@ decrypted_key(char *const pt_fs_key, size_t *const pt_fs_key_len,
 		ret = 0;
 		goto _return;
 	}
-	if (hash_authtok(fs_key_fp, cipher, authtok, hashed_authtok, iv) == 0) {
+	if (hash_authtok(fs_key_fp, cipher, authtok, hashed_authtok, iv) ==
+	    0) {
 		ret = 0;
 		goto _return;
 	}
@@ -159,7 +160,8 @@ decrypted_key(char *const pt_fs_key, size_t *const pt_fs_key_len,
 		ret = 0;
 		goto _return;
 	}
-	if (EVP_DecryptInit_ex(&ctx, cipher, NULL, hashed_authtok, iv) == 0) {
+	if (EVP_DecryptInit_ex(&ctx, cipher, NULL, hashed_authtok, iv) ==
+	    0) {
 		l0g("pam_mount: %s\n",
 		    "failed to initialize decryption code");
 		ret = 0;
@@ -168,7 +170,8 @@ decrypted_key(char *const pt_fs_key, size_t *const pt_fs_key_len,
 	/* assumes plaintexts is always <= ciphertext + EVP_MAX_BLOCK_LEN in length 
 	 * OpenSSL's documentation seems to promise this */
 	if (EVP_DecryptUpdate
-	    (&ctx, pt_fs_key, &segment_len, ct_fs_key, ct_fs_key_len) == 0) {
+	    (&ctx, pt_fs_key, &segment_len, ct_fs_key,
+	     ct_fs_key_len) == 0) {
 		l0g("pam_mount: %s\n", "failed to decrypt key");
 		ret = 0;
 		goto _return;
@@ -182,19 +185,20 @@ decrypted_key(char *const pt_fs_key, size_t *const pt_fs_key_len,
 		goto _return;
 	}
 	*pt_fs_key_len += segment_len;
-_return:
+      _return:
 	if (fclose(fs_key_fp) != 0) {
 		l0g("pam_mount: error closing file pointer\n");
 		ret = 0;
 	}
-_return_no_close:
+      _return_no_close:
 	if (EVP_CIPHER_CTX_cleanup(&ctx) == 0) {
 		l0g("pam_mount: error cleaning up cipher context\n");
 		ret = 0;
 	}
 
 	/* pt_fs_key_len is unsigned */
-	assert(ret == 0 || *pt_fs_key_len <= MAX_PAR + EVP_MAX_BLOCK_LENGTH);
+	assert(ret == 0
+	       || *pt_fs_key_len <= MAX_PAR + EVP_MAX_BLOCK_LENGTH);
 
 	return ret;
 #else
@@ -228,22 +232,29 @@ static void log_output(int fd)
  * NOTE: this fn simply runs lsof on a directory and logs its output for
  * debugging purposes
  */
-static void run_lsof(const struct config_t *const config, fmt_ptrn_t *vinfo)
+static void run_lsof(const struct config_t *const config,
+		     fmt_ptrn_t * vinfo)
 {
 	int i, _argc = 0, cstdout = -1, child_exit;
 	char *_argv[MAX_PAR + 1];
+	GError *err = NULL;
 	pid_t pid;
 	if (!config->command[0][LSOF])
 		l0g("pam_mount: lsof not defined in pam_mount.conf\n");
 	/* FIXME: NEW */
 	for (i = 0; config->command[i][LSOF]; i++)
-		add_to_argv(_argv, &_argc, config->command[i][LSOF], vinfo);
-	if ((pid =
-	     procopen(_argv[0], &_argv[1], 0, NULL, &cstdout, NULL)) == -1)
+		add_to_argv(_argv, &_argc, config->command[i][LSOF],
+			    vinfo);
+	if (g_spawn_async_with_pipes
+	    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
+	     &pid, NULL, &cstdout, NULL, &err) == FALSE) {
+		l0g("pam_mount: %s\n", err->message);
+		g_error_free(err);
 		return;
+	}
 	w4rn("pam_mount: lsof output (should be empty)...\n");
 	log_output(cstdout);
-	w4rn("pam_mount: %s\n", "waiting for mount");
+	w4rn("pam_mount: %s\n", "waiting for lsof");
 	if (waitpid(pid, &child_exit, 0) == -1)
 		l0g("pam_mount: error waiting for child\n");
 	CLOSE(cstdout);
@@ -290,7 +301,7 @@ static int mkmountpoint(vol_t * const volume, const char *const d)
 		goto _return;
 	}
 	volume->created_mntpt = TRUE;
-_return:
+      _return:
 	g_free(parent);
 	return ret;
 }
@@ -323,43 +334,42 @@ static int already_mounted(const struct config_t *const config,
 	assert(config_t_valid(config));
 
 	memset(match, 0x00, sizeof(match));
-	if (config->volume[vol].type == SMBMOUNT) {
+	if (config->volume[vol].type == SMBMOUNT
+	    || config->volume[vol].type == CIFSMOUNT) {
 		strcpy(match, "//");
 		strncat(match, config->volume[vol].server,
 			PATH_MAX - strlen(match));
 		strncat(match, "/", PATH_MAX - strlen(match));
 		strncat(match, config->volume[vol].volume,
 			PATH_MAX - strlen(match));
-	} else if (config->volume[vol].type == CIFSMOUNT) {
-		strncpy(match, config->volume[vol].server,
-			PATH_MAX - strlen(match));
-		strncat(match, "/", PATH_MAX - strlen(match));
-		strncat(match, config->volume[vol].volume,
-			PATH_MAX - strlen(match));
 	} else if (config->volume[vol].type == NCPMOUNT) {
 		strncpy(match, config->volume[vol].server,
 			PATH_MAX - strlen(match));
 		strncat(match, "/", PATH_MAX - strlen(match));
 		/* FIXME: volume sanity check in readconfig.c ensures optlist_value() will not return NULL for user */
-		strncat(match, optlist_value(config->volume[vol].options, "user"), PATH_MAX - strlen(match));
+		strncat(match,
+			optlist_value(config->volume[vol].options, "user"),
+			PATH_MAX - strlen(match));
 	} else if (config->volume[vol].type == NFSMOUNT) {
 		strncpy(match, config->volume[vol].server,
 			PATH_MAX - strlen(match));
 		strncat(match, ":", PATH_MAX - strlen(match));
 		strncat(match, config->volume[vol].volume,
 			PATH_MAX - strlen(match));
-	/* FIXME: ugly hack to support umount.crypt script.  I hope that
-         * util-linux will have native dm_crypt support some day */
-	} else if (config->volume[vol].type  == CRYPTMOUNT ) {
+		/* FIXME: ugly hack to support umount.crypt script.  I hope that
+		 * util-linux will have native dm_crypt support some day */
+	} else if (config->volume[vol].type == CRYPTMOUNT) {
 		int i;
 		char escaped_vol[PATH_MAX + 1];
 		strncpy(match, "/dev/mapper/", PATH_MAX - strlen(match));
 		/* isn't there a function to do this in libc or glib? */
-		for (i = 0; config->volume[vol].volume && i < PATH_MAX; i++) {
+		for (i = 0; config->volume[vol].volume && i < PATH_MAX;
+		     i++) {
 			if (config->volume[vol].volume[i] == '/')
-				escaped_vol[i]  = '_';
+				escaped_vol[i] = '_';
 			else
-				escaped_vol[i] = config->volume[vol].volume[i];
+				escaped_vol[i] =
+				    config->volume[vol].volume[i];
 		}
 		strncat(match, escaped_vol, PATH_MAX - strlen(match));
 	} else {
@@ -391,6 +401,7 @@ static int already_mounted(const struct config_t *const config,
 #elif defined(__FreeBSD__) || defined(__OpenBSD__)
 	{
 		FILE *fp;
+		GError *err = NULL;
 		int i, _argc = 0, cstdout = -1;
 		char *_argv[MAX_PAR + 1], dev[BUFSIZ + 1];
 		/*
@@ -405,10 +416,13 @@ static int already_mounted(const struct config_t *const config,
 		for (i = 0; config->command[i][MNTCHECK]; i++)
 			add_to_argv(_argv, &_argc,
 				    config->command[i][MNTCHECK], vinfo);
-		if ((pid =
-		     procopen(_argv[0], &_argv[1], 0, NULL, &cstdout,
-			      NULL)) == -1)
+		if (g_spawn_async_with_pipes
+		    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL,
+		     NULL, &pid, NULL, &cstdout, NULL, &err) == FALSE) {
+			l0g("pam_mount: %s\n", err->message);
+			g_error_free(err);
 			return -1;
+		}
 		fp = fdopen(cstdout, "r");
 		while (fgets(dev, BUFSIZ, fp)) {
 			/*
@@ -488,8 +502,9 @@ static void log_pm_input(const struct config_t *const config,
 
 /* ============================ do_unmount () ============================== */
 int
-do_unmount(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
-	   const char *const password, const gboolean mkmntpoint)
+do_unmount(struct config_t *config, const unsigned int vol,
+	   fmt_ptrn_t * vinfo, const char *const password,
+	   const gboolean mkmntpoint)
 /* PRE:    config points to a valid struct config_t*
  *         config->volume[vol] is a valid struct vol_t
  *         vinfo is a valid struct fmt_ptrn_t
@@ -498,7 +513,8 @@ do_unmount(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
  * FN VAL: if error 0 else 1, errors are logged
  */
 {
-	int i, child_exit, _argc = 0, ret = 1;
+	GError *err = NULL;
+	int i, child_exit, _argc = 0, ret = 1, cstderr = -1;
 	pid_t pid = -1;
 	char *_argv[MAX_PAR + 1];
 
@@ -514,20 +530,27 @@ do_unmount(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
 		run_lsof(config, vinfo);
 	/* FIXME: NEW */
 	for (i = 0; config->command[i][UMOUNT]; i++)
-		add_to_argv(_argv, &_argc, config->command[i][UMOUNT], vinfo);
+		add_to_argv(_argv, &_argc, config->command[i][UMOUNT],
+			    vinfo);
 	/* FIXME: ugly hack to support umount.crypt script.  I hope that
-         * util-linux will have native dm_crypt support some day */
-	if (config->volume[vol].type  == CRYPTMOUNT ) {
+	 * util-linux will have native dm_crypt support some day */
+	if (config->volume[vol].type == CRYPTMOUNT) {
 		_argc = 0;
 		add_to_argv(_argv, &_argc, "/usr/bin/umount.crypt", vinfo);
-		add_to_argv(_argv, &_argc, "umount.crypt", vinfo);
 		add_to_argv(_argv, &_argc, "%(MNTPT)", vinfo);
 	}
-	if ((pid =
-	     procopen(_argv[0], &_argv[1], 1, NULL, NULL, NULL)) == -1) {
+	if (g_spawn_async_with_pipes
+	    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, setrootid, NULL,
+	     &pid, NULL, NULL, &cstderr, &err) == FALSE) {
+		l0g("pam_mount: %s\n", err->message);
+		g_error_free(err);
 		ret = 0;
 		goto _return;
 	}
+	w4rn("pam_mount: umount errors (should be empty):\n");
+	log_output(cstderr);
+	CLOSE(cstderr);
+	w4rn("pam_mount: %s\n", "waiting for umount");
 	if (waitpid(pid, &child_exit, 0) == -1) {
 		l0g("pam_mount: error waiting for child\n");
 		ret = 0;
@@ -536,7 +559,7 @@ do_unmount(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
 		/* pass on through the result from the umount process */
 		ret = !WEXITSTATUS(child_exit);
 	}
-_return:
+      _return:
 	if (mkmntpoint != 0 && config->volume[vol].created_mntpt == TRUE) {
 		if (rmdir(config->volume[vol].mountpoint) == -1)	/* non-fatal */
 			w4rn("pam_mount: could not remove %s\n",
@@ -545,10 +568,43 @@ _return:
 	return ret;
 }
 
+/* ============================ pipewrite () =============================== */ 
+/* INPUT: fd, a valid file descriptor; buf, a buffer of size count
+ * SIDE AFFECTS: buf is written to fd
+ * OUTPUT: number of bytes written or 0 on error
+ * NOTE: SIGPIPE is ignored during this operation to avoid "broken pipe"
+ */
+int pipewrite(int fd, const void *buf, size_t count)
+{
+	int fnval;
+	struct sigaction ignoresact = {
+		.sa_handler = SIG_IGN
+	}, oldsact;
+
+	assert(fd >= 0);
+	assert(buf != NULL);
+	assert(count >= 0);
+
+	/* avoid bomb on command exiting before data written */
+	if (sigaction(SIGPIPE, &ignoresact, &oldsact) < 0) {
+		fnval = -1;
+		goto _return;
+	}
+	fnval = write(fd, buf, count);
+	/* restore old handler */
+	if (sigaction(SIGPIPE, &oldsact, NULL) < 0) {
+		fnval = -1;
+		goto _return;
+	}
+_return:
+	return fnval;
+}
+
 /* ============================ do_losetup () ============================== */
 int
-do_losetup(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
-	   const unsigned char *password, size_t password_len)
+do_losetup(struct config_t *config, const unsigned int vol,
+	   fmt_ptrn_t * vinfo, const unsigned char *password,
+	   size_t password_len)
 /* PRE:    config points to a valid struct config_t*
  *         config->volume[vol] is a valid struct vol_t
  *         vinfo is a valid struct fmt_ptrn_t
@@ -558,6 +614,7 @@ do_losetup(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
  */
 {
 	pid_t pid;
+	GError *err = NULL;
 	int i, child_exit, _argc = 0, cstdin = -1, cstderr = -1;
 	char *_argv[MAX_PAR + 1];
 	const char *cipher =
@@ -587,17 +644,25 @@ do_losetup(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
 			    config->command[i][LOSETUP], vinfo);
 	}
 	if (cipher) {
-		if ((pid =
-		     procopen(_argv[0], &_argv[1], 1, &cstdin, NULL, &cstderr)) == -1)
-			return 0;
-		if (write(cstdin, password, password_len) != password_len) {
-			l0g("pam_mount: error sending password to losetup\n");
+		if (g_spawn_async_with_pipes
+		    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL,
+		     NULL, &pid, &cstdin, NULL, &cstderr, &err) == FALSE) {
+			l0g("pam_mount: %s\n", err->message);
+			g_error_free(err);
 			return 0;
 		}
+		if (pipewrite(cstdin, password, password_len) != password_len)
+			/* FIXME: clean: returns value of exit below */
+			l0g("pam_mount: error sending password to losetup\n");
 		CLOSE(cstdin);
 	} else {
-		if ((pid = procopen(_argv[0], &_argv[1], 1, NULL, NULL, &cstderr)) == -1)
+		if (g_spawn_async_with_pipes
+		    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL,
+		     NULL, &pid, NULL, NULL, &cstderr, &err) == FALSE) {
+			l0g("pam_mount: %s\n", err->message);
+			g_error_free(err);
 			return 0;
+		}
 	}
 	log_output(cstderr);
 	w4rn("pam_mount: %s\n", "waiting for losetup");
@@ -610,7 +675,7 @@ do_losetup(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
 }
 
 /* ============================ do_unlosetup () ============================ */
-int do_unlosetup(struct config_t *config, fmt_ptrn_t *vinfo)
+int do_unlosetup(struct config_t *config, fmt_ptrn_t * vinfo)
 /* PRE:    config points to a valid struct config_t*
  *         vinfo is a valid struct fmt_ptrn_t
  * POST:   volume has associated with a loopback device
@@ -618,6 +683,7 @@ int do_unlosetup(struct config_t *config, fmt_ptrn_t *vinfo)
  */
 {
 	pid_t pid;
+	GError *err = NULL;
 	char *_argv[MAX_PAR + 1];
 	int i, child_exit, _argc = 0;
 
@@ -633,9 +699,13 @@ int do_unlosetup(struct config_t *config, fmt_ptrn_t *vinfo)
 	for (i = 0; config->command[i][UNLOSETUP]; i++)
 		add_to_argv(_argv, &_argc,
 			    config->command[i][UNLOSETUP], vinfo);
-	if ((pid =
-	     procopen(_argv[0], &_argv[1], 1, NULL, NULL, NULL)) == -1)
+	if (g_spawn_async_with_pipes
+	    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
+	     &pid, NULL, NULL, NULL, &err) == FALSE) {
+		l0g("pam_mount: %s\n", err->message);
+		g_error_free(err);
 		return 0;
+	}
 	w4rn("pam_mount: %s\n", "waiting for losetup delete");
 	waitpid(pid, &child_exit, 0);
 	/* pass on through the result */
@@ -644,8 +714,9 @@ int do_unlosetup(struct config_t *config, fmt_ptrn_t *vinfo)
 
 /* ============================ check_filesystem () ======================== */
 static int
-check_filesystem(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo,
-		 const unsigned char *password, size_t password_len)
+check_filesystem(struct config_t *config, const unsigned int vol,
+		 fmt_ptrn_t * vinfo, const unsigned char *password,
+		 size_t password_len)
 /* PRE:    config points to a valid struct config_t*
  *         config->volume[vol] is a valid struct vol_t
  *         vinfo is a valid struct fmt_ptrn_t
@@ -655,6 +726,7 @@ check_filesystem(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vi
 {
 #if defined (__linux__)
 	pid_t pid;
+	GError *err = NULL;
 	int i, child_exit, _argc = 0, cstdout = -1, cstderr = -1;
 	char *_argv[MAX_PAR + 1];
 	char *fsck_target =
@@ -671,19 +743,27 @@ check_filesystem(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vi
 		return 0;
 	}
 	if (optlist_exists(config->volume[vol].options, "loop")) {
-		if (!do_losetup(config, vol, vinfo, password, password_len))
+		if (!do_losetup
+		    (config, vol, vinfo, password, password_len))
 			return 0;
 		fsck_target = config->fsckloop;
 	} else
 		w4rn("pam_mount: volume not a loopback (options: %s)\n",
 		     optlist_to_str(options, config->volume[vol].options));
 	/* FIXME: NEW */
+	/* FIXME: need to fsck /dev/mapper/whatever... */
+	fmt_ptrn_update_kv(vinfo, "FSCKTARGET", fsck_target);
 	for (i = 0; config->command[i][FSCK]; i++)
-		add_to_argv(_argv, &_argc, config->command[i][FSCK], vinfo);
-	if ((pid =
-	     procopen(_argv[0], &_argv[1], 1, NULL, &cstdout, &cstderr)) == -1)
+		add_to_argv(_argv, &_argc, config->command[i][FSCK],
+			    vinfo);
+	if (g_spawn_async_with_pipes
+	    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
+	     &pid, NULL, &cstdout, &cstderr, &err) == FALSE) {
+		l0g("pam_mount: %s\n", err->message);
+		g_error_free(err);
 		return 0;
-	log_output(cstdout); /* stdout and stderr most be logged for fsck */
+	}
+	log_output(cstdout);	/* stdout and stderr most be logged for fsck */
 	log_output(cstderr);
 	CLOSE(cstderr);
 	w4rn("pam_mount: %s\n", "waiting for filesystem check");
@@ -704,7 +784,8 @@ check_filesystem(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vi
 
 /* ============================ do_mount () ================================ */
 int
-do_mount(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo, const char *password,
+do_mount(struct config_t *config, const unsigned int vol,
+	 fmt_ptrn_t * vinfo, const char *password,
 	 const gboolean mkmntpoint)
 /* PRE:    config points to a valid struct config_t*
  *         config->volume[vol] is a valid struct vol_t
@@ -735,14 +816,16 @@ do_mount(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo, con
 			return 0;
 		} else
 		    if (strcmp
-			(prev_mntpt, config->volume[vol].mountpoint) == 0) {
+			(prev_mntpt,
+			 config->volume[vol].mountpoint) == 0) {
 			w4rn("pam_mount: %s already seems to be mounted at %s, skipping\n", config->volume[vol].volume, prev_mntpt);
 			return 1;
 		} else {
 			w4rn("pam_mount: %s already mounted elsewhere at %s\n", config->volume[vol].volume, prev_mntpt);
-		/* FIXME: ugly hack to support umount.crypt script.  I hope that
-		 * util-linux will have native dm_crypt support some day */
-			if (config->volume[vol].type != LCLMOUNT && config->volume[vol].type != CRYPTMOUNT)
+			/* FIXME: ugly hack to support umount.crypt script.  I hope that
+			 * util-linux will have native dm_crypt support some day */
+			if (config->volume[vol].type != LCLMOUNT
+			    && config->volume[vol].type != CRYPTMOUNT)
 				mount_again = 0;
 		}
 	if (!exists(config->volume[vol].mountpoint))
@@ -756,6 +839,7 @@ do_mount(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo, con
 			return 0;
 		}
 	if (mount_again) {
+		GError *err = NULL;
 		if (!config->command[0][MNTAGAIN]) {
 			l0g("pam_mount: mntagain not defined in pam_mount.conf\n");
 			return 0;
@@ -766,10 +850,14 @@ do_mount(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo, con
 		for (i = 0; config->command[i][MNTAGAIN]; i++)
 			add_to_argv(_argv, &_argc,
 				    config->command[i][MNTAGAIN], vinfo);
-		if ((pid =
-		     procopen(_argv[0], &_argv[1], 1, NULL, NULL,
-			      &cstderr)) == -1)
+		if (g_spawn_async_with_pipes
+		    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD,
+		     setrootid, NULL, &pid, NULL, NULL, &cstderr,
+		     &err) == FALSE) {
+			l0g("pam_mount: %s\n", err->message);
+			g_error_free(err);
 			return 0;
+		}
 #else
 		/* FIXME */
 		l0g("pam_mount: %s\n",
@@ -777,6 +865,7 @@ do_mount(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo, con
 		return 0;
 #endif
 	} else {
+		GError *err = NULL;
 		if (!config->command[0][config->volume[vol].type]) {
 			l0g("pam_mount: proper mount command not defined in pam_mount.conf\n");
 			return 0;
@@ -808,42 +897,55 @@ do_mount(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo, con
 		     "about to start building mount command");
 		/* FIXME: NEW */
 		/* FIXME:
-			l0g("pam_mount: %s\n",
-			    "config->volume[vol].type is unknown");
-			return 0;
-		*/
-		for (i = 0; config->command[i][config->volume[vol].type]; i++)
+		   l0g("pam_mount: %s\n",
+		   "config->volume[vol].type is unknown");
+		   return 0;
+		 */
+		for (i = 0; config->command[i][config->volume[vol].type];
+		     i++)
 			add_to_argv(_argv, &_argc,
-				    config->command[i][config-> volume[vol].
-							   type], vinfo);
-		if (config->volume[vol].type == LCLMOUNT && !check_filesystem
-			    (config, vol, vinfo, _password, _password_len))
-				l0g("pam_mount: %s\n",
-				    "error checking filesystem but will continue");
+				    config->command[i][config->volume[vol].
+						       type], vinfo);
+		 if (config->volume[vol].type == LCLMOUNT
+		    && !check_filesystem(config, vol, vinfo, _password,
+					 _password_len))
+			l0g("pam_mount: %s\n",
+			    "error checking filesystem but will continue");
 		/* send password down pipe to mount process */
-		if (config->volume[vol].type == SMBMOUNT)
+		if (config->volume[vol].type == SMBMOUNT
+		    || config->volume[vol].type == CIFSMOUNT)
 			setenv("PASSWD_FD", "0", 1);
-		if ((pid =
-		     procopen(_argv[0], &_argv[1], 1, &cstdin, NULL,
-			      &cstderr)) == -1)
+		if (g_spawn_async_with_pipes
+		    (NULL, _argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD,
+		     setrootid, NULL, &pid, &cstdin, NULL, &cstderr,
+		     &err) == FALSE) {
+			l0g("pam_mount: %s\n", err->message);
+			g_error_free(err);
 			return 0;
+		}
 		if (config->volume[vol].type != NFSMOUNT) {
-			if (write(cstdin, _password, _password_len) != _password_len) {
+			if (pipewrite(cstdin, _password, _password_len) !=
+			    _password_len)
+				/* FIXME: clean: returns value of exit below */
 				l0g("pam_mount: error sending password to mount\n");
-				return 0;
-			}
 			CLOSE(cstdin);
 		}
+
 	}
+_return:
 	/* Paranoia? */
 	memset(_password, 0x00, MAX_PAR + EVP_MAX_BLOCK_LENGTH);
 	w4rn("pam_mount: mount errors (should be empty):\n");
 	log_output(cstderr);
 	CLOSE(cstderr);
 	w4rn("pam_mount: %s\n", "waiting for mount");
-	waitpid(pid, &child_exit, 0);
-	/* pass on through the result from the mount process */
-	return !WEXITSTATUS(child_exit);
+	if (waitpid(pid, &child_exit, 0) == -1) {
+		l0g("pam_mount: error waiting for child\n");
+		return 0;
+	} else {
+		/* pass on through the result from the umount process */
+		return !WEXITSTATUS(child_exit);
+	}
 }
 
 /* ============================ mount_op () ================================ */
@@ -861,28 +963,21 @@ do_mount(struct config_t *config, const unsigned int vol, fmt_ptrn_t *vinfo, con
  *         ** checked by read_volume()
  */
 int mount_op(int (*mnt)
-	      (struct config_t * config, const unsigned int vol, fmt_ptrn_t *vinfo,
-	       const char *password, const int mkmntpoint),
-	     struct config_t *config,
-	     const unsigned int vol, const char *password, const int mkmntpoint)
+	      (struct config_t * config, const unsigned int vol,
+	       fmt_ptrn_t * vinfo, const char *password,
+	       const int mkmntpoint), struct config_t *config,
+	     const unsigned int vol, const char *password,
+	     const int mkmntpoint)
 {
 	int fnval;
 	fmt_ptrn_t vinfo;
 	char options[MAX_PAR + 1];
-	struct sigaction sact, oldsact;
 
 	assert(config_t_valid(config));
 
-	/* avoid bomb on mount command exiting before password written */
-	sact.sa_handler = SIG_DFL;
-	sact.sa_flags = 0;
-	if (sigaction(SIGPIPE, &sact, &oldsact) < 0) {
-		fnval = 0;
-		goto _nosigact_return;
-	}
-
 	fmt_ptrn_init(&vinfo);
-	fmt_ptrn_update_kv(&vinfo, "MNTPT", config->volume[vol].mountpoint);
+	fmt_ptrn_update_kv(&vinfo, "MNTPT",
+			   config->volume[vol].mountpoint);
 	fmt_ptrn_update_kv(&vinfo, "FSCKLOOP", config->fsckloop);
 	fmt_ptrn_update_kv(&vinfo, "VOLUME", config->volume[vol].volume);
 	fmt_ptrn_update_kv(&vinfo, "SERVER", config->volume[vol].server);
@@ -894,11 +989,9 @@ int mount_op(int (*mnt)
 
 	if (debug)
 		log_pm_input(config, vol);
-	
+
 	fnval = mnt(config, vol, &vinfo, password, mkmntpoint);
 	fmt_ptrn_close(&vinfo);
-_return:
-	sigaction(SIGPIPE, &oldsact, NULL);
-_nosigact_return:
+      _return:
 	return fnval;
 }
